require 'spec_helper'

class String
  def |
    gsub(/^\s*\|/,'').strip
  end
end

describe Post do
  
  let(:path) { '/posts/2010-12-26-test.textile' }
  
  describe 'filesystem reading' do
    it 'should remove bom from start of file so title can be read' do
      post = Post.from_file 'spec/2011-04-09-bom-removal-test.textile'
      post[:title].should == 'Test title'
    end
    
    it 'should read title correctly in bom-less file' do
      post = Post.from_file 'spec/2011-04-09-without-bom-test.textile'
      post[:title].should == 'Test title'
    end
  end
  
  describe 'parsing' do
    it 'should not require any header section' do
      raw = %{
        |just some body text
      }.|
      post = Post.from_s raw
      post[:title].should_not be
      post[:date].should == Date.today
    end
  
    it 'should load post properties from header' do
      raw = %{
        |title: test title
        |tags: [post, gift, fun]
        |---
        |test body
      }.|
      post = Post.from_s raw
      post[:title].should == 'test title'
      post[:tags].should == ['post', 'gift', 'fun']
    end
  
    it 'should render the textile under the header as html' do
      raw = %{
        |title: test
        |---
        |body
      }.|
      post = Post.from_s raw
      post.html.should == '<p>body</p>'
    end
  
    it 'should render all textile when no header' do
      raw = %{
        |body
      }.|
      post = Post.from_s raw
      post.html.should == '<p>body</p>'
    end
    
    it 'should load meta data when no body' do
      raw = %{
        |title: test title
        |tags: [post, gift, fun]
        |---
      }.|
      post = Post.from_s raw
      post[:title].should == 'test title'
      post[:tags].should == ['post', 'gift', 'fun']
    end
  end
  
  describe 'finding' do
    it 'should glob for files under the base dir' do
      Dir.should_receive(:glob).with 'posts/**/*.textile'
      Post.base_dir = 'posts'
      Post.glob = '**/*.textile'
      Post.filenames
    end
  
    it 'should load posts sorted most recent first' do
      paths = [
        '/posts/2010-12-07-test-post2.textile',
        '/posts/2010-12-09-test-post1.textile',
        '/posts/2010-05-18-test-post3.textile'
      ]
      Post.stub!(:filenames).and_return(paths)
      IO.stub!(:read).and_return("title: test\n\nbody")
      posts = Post.find
      posts.map { |post| post[:date] }.should == [
        Date.parse('2010-12-09'),
        Date.parse('2010-12-07'),
        Date.parse('2010-05-18')
      ]
    end
  
    [:find_latest, :find_single].each do |method|
      it "should load the latest post through #{method}" do
        paths = [
          '/posts/2010-12-07-test-post2.textile',
          '/posts/2010-12-09-test-post1.textile',
          '/posts/2010-05-18-test-post3.textile'
        ]
        Post.stub!(:filenames).and_return(paths)
        IO.stub!(:read).and_return("title: test\n\nbody")
        latest = Post.send(method)
        latest[:date].should == Date.parse('2010-12-09')
      end
    end
  end
  
  describe 'templates' do
    it 'should determine template based on post directory' do
      IO.stub!(:read).and_return 'body'
      Post.from_file('/posts/2010-01-01-blog-post.textile')[:template].should_not be
      Post.from_file('/posts/blog/2010-01-01-blog-post.textile')[:template].should == :blog_post
      Post.from_file('/posts/daily_gift_ideas/2010-01-01-blog-post.textile')[:template].should == :daily_gift_ideas_post
    end
  end

  describe 'slugs' do
    it 'should convert path to slug' do
      IO.stub!(:read).and_return 'body'
      post = Post.from_file '/posts/2010-12-07-test-post2.textile'
      post.slug.should == '2010/12/07/test-post2'
    end
  
    it 'should have # slug if created from string instead of path' do
      post = Post.from_s ''
      post.slug.should == '#'
    end
  end

  describe 'tags' do
    before :each do
      posts = [['sharing'], ['social'], ['fun', 'technology'], ['technology'], ['technology']].map do |tags|
        Post.new({ 'tags' => tags }, 'body', Date.parse('2010-12-09'))
      end
      Post.stub!(:find).and_return(posts)
    end
    
    it 'should return hash of tags with count' do
      Post.categories.should =~ [['sharing', 1], ['fun', 1], ['social', 1], ['technology', 3]]
    end
    
    it 'should handle no tags' do
      Post.stub!(:find).and_return([Post.new({}, 'body', Date.parse('2010-12-09'))])
      Post.categories.should be_empty
      Post.find_by_tag('fun').should be_empty
    end
    
    it 'should sort categories alphabetically' do
      Post.categories.should =~ [['fun', 1], ['sharing', 1], ['social', 1], ['technology', 3]]
    end
    
    it 'should filter posts by tag' do
      Post.find_by_tag('technology').should have(3).items
    end
    
    it 'should filter case insensitively' do
      Post.find_by_tag('Technology').should have(3).items
    end
  end
end